@@ -53,6 +53,36 @@ __PACKAGE__->has_many(
   { 'foreign.person_id' => 'self.id' }
 );
 
+__PACKAGE__->has_many(
+  posts =>
+  'Example::Schema::Result::Post',
+  { 'foreign.person_id' => 'self.id' }
+);
+
+__PACKAGE__->has_many(
+  comments =>
+  'Example::Schema::Result::Comment',
+  { 'foreign.person_id' => 'self.id' }
+);
+
+__PACKAGE__->has_many(
+  viewable_posts2 =>
+    'Example::Schema::Result::Post::Viewable',
+    sub {
+      my $args = shift;
+      return 
+        +{ },
+         { "$args->{foreign_alias}.id" => \[' is not null', $args->{self_result_object}->id ] };
+    },
+);
+
+sub viewable_posts($self, $search_args = {}) {
+  my $schema = $self->result_source->schema;
+  return $schema->resultset('Post::Viewable')->search(
+    $search_args,
+    { bind => [ $self->id ] }
+  );
+}
 
 __PACKAGE__->validates(username => presence=>1, length=>[3,24], format=>'alpha_numeric', unique=>1);
 __PACKAGE__->validates( password => (presence=>1, confirmation => 1,  on=>'create' ));
@@ -73,6 +103,11 @@ __PACKAGE__->accept_nested_for('contacts', {allow_destroy=>1});
 
 __PACKAGE__->accept_nested_for('profile');
 
+__PACKAGE__->validates_with(sub {
+  my ($self, $opts) = @_;
+  $self->errors->add(undef, "No MEga!!!!!") if (($self->last_name||'') eq 'mega');
+});
+
 sub validate_roles($self, $attribute_name, $value, $opt) {
 
   # Tricky since you want to use the cached rows.  This could likely be optimized
@@ -146,4 +181,22 @@ sub update_account($self, $request) {
   return $self->valid;
 }
 
+## There's are proxied to other resultsets for now but we expect that
+## ecentually they could be impacted by the current user.
+
+sub states {
+  my $self = shift;
+  return $self->result_source->schema->resultset('State');
+}
+
+sub viewable_roles {
+  my $self = shift;
+  return $self->result_source->schema->resultset('Role');
+}
+
+sub employment_options {
+  my $self = shift;
+  return $self->result_source->schema->resultset('Employment');
+}
+
 1;
